// Copyright 2024 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

//go:build ignore

package main

import (
	"bufio"
	"bytes"
	"flag"
	"log"
	"os"
	"strings"
)

func main() {
	output := flag.String("output", "", "output file name")
	flag.Parse()
	if *output == "" {
		log.Fatal("output file name is required")
	}
	out, err := os.Create(*output)
	if err != nil {
		log.Fatal(err)
	}
	defer func() {
		if err := out.Close(); err != nil {
			log.Fatal(err)
		}
	}()
	out.WriteString("// Code generated by syscalltrace_generate_windows.go. DO NOT EDIT.\n\n")
	out.WriteString("package runtime\n\n")
	out.WriteString("var staticSyscalls = [...]string{\n")
	for _, file := range flag.Args() {
		content, err := os.ReadFile(file + ".go")
		if err != nil {
			log.Fatal(err)
		}
		scanner := bufio.NewScanner(bytes.NewReader(content))
		for scanner.Scan() {
			if after, found := strings.CutPrefix(scanner.Text(), "//go:cgo_import_dynamic runtime._"); found {
				name, _, _ := strings.Cut(after, " ")
				out.WriteString("\t\"" + name + `\x00"` + ",\n")
			}
		}
	}
	out.WriteString("}\n")
}
